/***************************************************************************
 *   Copyright (C) 2004 by Abdiel Janulgue                                 *
 *   xynopsis at users dot sourceforge dot net                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
// kiosklock.h: interface for the KioskLock class.
//
//////////////////////////////////////////////////////////////////////

#ifndef __KIOSK_LOCK_H_
#define __KIOSK_LOCK_H_

#include <qwidget.h>
#include <qobject.h>
#include <qmessagebox.h>
#include <qpixmap.h>
#include "xmlrpc++/XmlRpc.h"
#include <qtimer.h>
#include <sstream>
#include <iostream>

#include "extenddlg.h"
#include "alabel.h"
#include "kiosklockiface.h"

using namespace XmlRpc;

// forward decls
class SerialDialog;
class ALabel;
class ChangePassDlg;
class AppSelector;
class KioskLock;

class ExtendDlg: public Extend
{
        Q_OBJECT
public:
        ExtendDlg(KioskLock* kl);
        bool active() const;

public:
        int exec();

protected slots:
        void accept();
        void reject();

private:
        KioskLock *lockinst;
        bool isOpen;
};

class InfoLabel : public ALabel
{
        Q_OBJECT
public:
        InfoLabel(QWidget *parent);
        void setInfoText(const char* text, int duration = 10);
        void setWarning(const QString& message, const char *title = "Warning:");

private:
        static std::string formatText(std::string s, int wid = 35);

private slots:
        void hidewindow();

private:
        QTimer animt;
};

class TimerLabel: public ALabel
{
        Q_OBJECT
public:
        TimerLabel(QWidget *parent);        
        void startFlasher();
        void stopFlasher();

private slots:
        void flash();
        void startoff();
        void  off();

private:
        bool timer_started;
        QTimer flasher;
        QTimer oftimer;
        QTimer flasher_i;
};

class KioskLock : public QWidget, public KioskLockInterface
{
        friend class ChangePassDlg;
        friend class ExtendDlg;
        
        Q_OBJECT
 public:
        KioskLock(QWidget* parent, const char * name);
        ~KioskLock();
        
        // user-configurable options
        void setServer(const QString &host, int port);
        void setClientInfo(const QString &);
        void setWarnings(int init_warn, int final_warn);
        void setClientPort(int);

 public slots:
        int gettime(const char*, const char *pass = "0", bool peek = false);
        bool invalidate();
        bool update(int minutes, const char* newserial = "0");
        void reloadstate();
        void checkTime();
        void changePassword();
        void reload();
        void about();
        void closeScreen();
        void openScreen(unsigned int, const char*, const char*);
        void lockinvalidate();
        void lockupdate();
        void aliveclient();
        void updateLogoff();
        void informUser(const char*, int duration = 0);
        void countdown();
        void savestate();
        void resetTimeout();
        void remainingcheck(bool warn, bool timerdisplay = false);
        void show();
        void hide();

	bool isNodeviewUp();

 public:
        void ErrorDialog(QWidget* p, const char*);
        void InfoDialog(QWidget* p, const char*);
        void logoffSwitch(bool enable);
        // For manual window arrangement
        void setTaskbarWidget(QWidget*);
	QString backgroundUrl() const;
	QString timerDisplay() const;

 signals:
        void connectionProgress(int);
        // True if member account logged in. False if not
        void memberStatus(bool);
        // Session has started. kioskmode is true if session is a
        // kiosk (restricted) session
        void sessionStart(bool kioskmode);
        void sessionEnd(bool kioskmode);
        void timerUpdated(const QString&);
        
 protected:    
        void paintEvent(QPaintEvent *);
        virtual bool x11Event(XEvent *);

 private slots:
        void connectProgress(int);
        void focusDialog();
    
 private:
        void getApplist();
        void stayOnTop();
        static void ungrabInput();
        static bool grabKeyboard();
        static bool grabMouse();
        static bool grabInput();
        static void fakeFocusIn( WId window );
        void msgBox( QMessageBox::Icon type, const QString &txt );
                
        SerialDialog *sdlg;
        TimerLabel *timerbox;
        InfoLabel *infobox;
        ChangePassDlg *cpdialog;
        ExtendDlg *extdlg;
        AppSelector *panel;
        
        QString serv_ip;
        QString serial;
        QString key;
        QString serv_uri;
        QString smessage;
        QString _station;
	QString bgurl;
	QString timer_counter;
        
        int Cx, Cy;
        int serv_port;
        int checkup_interval;
        int timeout_value;
        int client_port;
        unsigned int seconds;
        unsigned int init_sec;
        unsigned int timeoutval;
        unsigned int warning_init;
        unsigned int warning_final;
        size_t appsize;

        bool locked;
        bool timerbox_visible;
        bool is_member;
        bool is_emptypassword;
        bool kioskmode;
        QPixmap bmp;
        
        QTimer	savetm;
        QTimer	cdowntm;
        QTimer  updatetm;
};

// TODO: encapsulate in thread to prevent blocking. Requests together with
// corresponding helper methods should be pushed to a stack
// and popped back when data is available.
// Stack should be a map containing (a): Method name, (b) Address of a KioskLock::member
// function pointer that handles the returned XmlRpcClient.exec().
class XClient: public QObject, public XmlRpcClient
{
        Q_OBJECT
 public:
        XClient(KioskLock* kl, const char * host, int port, const char* uri = 0)
                :QObject(kl, "XClient"),
                XmlRpcClient(host, port, uri), lockinst(kl), shost(host){ }
	virtual ~XClient()
	{ close(); }
        
        unsigned  handleEvent (unsigned eventType) 
        {
                if(eventType == XmlRpcDispatch::Exception) {
                        std::ostringstream msg;
                        msg << "Cannot connect to server " << shost;
                        lockinst->informUser(msg.str().c_str());
                }
                switch(_connectionState) {
                        // TODO: run timer before write_request
                        // end timer after write_request
                        // run timer before read_header
                        // end timer after read_header
                case WRITE_REQUEST:
                        status(50);
                        break;
                case READ_HEADER:
                        status(100);
                        break;
                default:
                        break;
                }
                
                return XmlRpcClient::handleEvent(eventType);
        }
 signals:        
        void status(int);
  
 private:       
        KioskLock* lockinst;
        QString shost;
};

#endif //  __KIOSK_LOCK_H_
